;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x4e130a9a1f448bb17d7ae0541ac9ee46 . "./code.cg2js/cg-common-control.fasl")
(#x26f663e3ab479bd2ee42b0aaa8fddfc2 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x4561c81abef8b6391ad6e682b31deb56 . "./code.cg2js/cg-yes-no-string.fasl")
(#x11ab54aba7f669df5fe3212ac56bf7df . "./code.cg2js/cg-clipboard.fasl")
(#x63ce96ff8f36938278fcb94aa34dbfa0 . "./code.cg2js/cg-dde-utils.fasl")
(#x648ca19db2a1eacb3ebb24154dc6e0ec . "./code.cg2js/cg-editable-text.fasl")
(#xc9de56b53198d9cf9e0bc93ab9f320ad . "./code.cg2js/cg-player.fasl")
(#xdf895b5413c7a9562b7bce9d09682d45 . "./code.cg2js/cg-text-or-combo.fasl")
(#x82501eb4726ab2d2be05a5966a82c919 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x695fc671257a40acae3a30baf0107b6b . "./code.cg2js/cg-graphics-context.fasl")
(#x5ab7326e5ba07d66eeb74c18b8dbafc0 . "./code.cg2js/cg-choose-printer.fasl")
(#xd3d3725e53b880abb8528fb66566e2bd . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xeb85fd878fa6f682d83bd341148dd68c . "./code.cg2js/cg-edit-in-place.fasl")
(#x0264c3b11b6d1d099d590c228242ed35 . "./code.cg2js/cg-split-bar.fasl")
(#x7352526c2bdfaab9127f7c8f98550dfc . "./code.cg2js/cg-timer.fasl")
(#x115cd91a92bf6aa956ec09ba43575cc9 . "./code.cg2js/cg-progress-indicator.fasl")
(#x4ea8254e3825b777dd01e12e32e1f1f6 . "./code.cg2js/cg-object-editor.fasl")
(#xf04a005b8f8ad34bf63d6921eb9fd427 . "./code.cg2js/cg-fill-texture.fasl")
(#xdc92c4550c773b3e912a1885206469ed . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x26fd34974c84e07bb2f2173a811ec5ba . "./code.cg2js/cg-os-window.fasl")
(#x24b2b6c97a6e7f296dddff47814b9923 . "./code.cg2js/cg-color-dialog.fasl")
(#x61141c76873536ab43960818205e13f3 . "./code.cg2js/cg-combo-box.fasl")
(#x1d9bb126077aed72763e2d383ae544d4 . "./code.cg2js/cg-directory-dialog.fasl")
(#x2942be79268ad2673ba5f23c105bdd96 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x60c0a5eeb8d8373491fb7d2329438b99 . "./code.cg2js/cg-tooltip.fasl")
(#x9992bdc993cdd385c79bd7fbdf4a85e8 . "./code.cg2js/cg-lettered-menu.fasl")
(#x66cc715842459a908eb0606688d6c64b . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x6de3292cef1b72b00403e931d5a6703d . "./code.cg2js/cg-pixmap.fasl")
(#x0a3b2e2791e67721dcd43db9d7b39e32 . "./code.cg2js/cg.fasl")
(#x78131ec0f953ad0a200e94ed7118b365 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x097eba315534ed796313a64aa7477f55 . "./code.cg2js/cg-template-string.fasl")
(#x9223f687b1c322a73596821c54315d14 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xc529aa64111463a91b521fdb144511e5 . "./code.cg2js/cg-check-box.fasl")
(#xb78407c99f5eefff5a3efef01d6628c1 . "./code.cg2js/cg-toggling-widget.fasl")
(#xfaf472b3e3731a8d2f1aac1d0f3d2cb1 . "./code.cg2js/cg-radio-button.fasl")
(#x00a7ec21c34d8c21119b66cb02759e07 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x9ae1fb0fa9f4e52217285821b325aa2b . "./code.cg2js/cg-scaling-stream.fasl")
(#x9610bd921c5411497314f283c7340bfa . "./code.cg2js/cg-multi-picture-button.fasl")
(#x5e98d7f444efdfc34eb1913c773724c2 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x24e6efbadbf7ba409a835fe8cd5f54f0 . "./code.cg2js/cg-grid-widget.fasl")
(#x8ee50394deb54846e4077cc6425dac5c . "./code.cg2js/cg-text-edit-pane.fasl")
(#xd37916fdad50e6c19936d419acc29c30 . "./code.cg2js/cg-item-list.fasl")
(#xb0a795584dde2fc0cdae8a21291529a6 . "./code.cg2js/cg-paren-matching.fasl")
(#x39ca6b55a131a1dff61b311cca078052 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x91ea9c6db3852c4a573583bc794c421b . "./code.cg2js/cg-html-dialog.fasl")
(#x2ff362497a69329f0f74dcb68e0c46d3 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x01ee77c20dc1e3670b7e3de7acd8ed29 . "./code.cg2js/cg-project-window.fasl")
(#x7daaedd298cf0889f603a28488dc7025 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x81ca955d4ba0bfd140e06726bdddde74 . "./code.cg2js/cg-toolbar.fasl")
(#x5fd66d66d2f2cebfef1a7c031f0269d5 . "./code.cg2js/cg-caret.fasl")
(#x5e13ddd997007ed043bb71966baa7b7f . "./code.cg2js/cg-scroll-bar.fasl")
(#x82f7f6a7652a849f4dd7c0a78278fe09 . "./code.cg2js/cg-web-browser.fasl")
(#x7b85979a26ebe049aa251b9068746b17 . "./code.cg2js/cg-dropping-outline.fasl")
(#xf24977986e8fd281b710b1ea07b7df9c . "./code.cg2js/cg-font-dialog.fasl")
(#x7a64bbb4e9ff10dcd58d9e70495952f8 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x7a4d970721836ec4fa739a0171498c6c . "./code.cg2js/cg-yes-no-list.fasl")
(#x13d5df6629eeaa1634f4fcee93154336 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x5e44f192775d11fee7c4f358802c0ee7 . "./code.cg2js/cg-lamp.fasl")
(#x36956199340924bcc1452806b273100b . "./code.cg2js/cg-drawable.fasl")
(#xb73880b1ef239cae66c222423deb3d39 . "./code.cg2js/cg-choice-list.fasl")
(#xdeea80f96eba5627638a0405caa2e788 . "./code.cg2js/cg-curve.fasl")
(#x9221c8ea5d9eeccdce2503980266b396 . "./code.cg2js/cg-base.fasl")
(#x669648e0c99866e367ab45da4a4e7696 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x2188e9c09fc62d7a2ca65a4c0b34ee5e . "./code.cg2js/cg-printing.fasl")
(#xacf34bc09c8182473c65b467b503222e . "./code.cg2js/cg-chart-or-plot.fasl")
(#x701200506d4623109ce74d994f1ad095 . "./code.cg2js/cg-hotspot.fasl")
(#x14883d5278d013dd571ba857374fe5f8 . "./code.cg2js/cg-object-editor-layout.fasl")
(#xb6aae74a6dea9c1c963b1c95a067e6a6 . "./code.cg2js/cg-utility-dialog.fasl")
(#x0180388bd2ae414273d67f9e1a845681 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xf16660eb48d7a5b9fb97c1d16bc52b31 . "./code.cg2js/cg-comtab.fasl")
(#xa8277b656790a63552948b979112b73e . "./code.cg2js/cg-os-widget.fasl")
(#x0099aff0008bdcbc8e51a79a9da8c84d . "./code.cg2js/cg-icon.fasl")
(#xcc1a3c5c81ff02420cf248e82e6c674e . "./code.cg2js/cg-dev.fasl")
(#x24ac3408d643823dce50d3d76e02211a . "./code.cg2js/cg-calendar.fasl")
(#xb620b9a0e0fa96eba8c981da83d04b78 . "./code.cg2js/cg-group-box.fasl")
(#x5ff83006c8fdd16db4ae86543b8a13bd . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x7407b886333d401a184386f812aa97e6 . "./code.cg2js/cg-bitmap-pane.fasl")
(#xf3c498d50d76da87cb442320f4ec7642 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x1888ae2e8642bfe92bdcd64e7c69f7dd . "./code.cg2js/cg-nodes.fasl")
(#x80954c7674b6962a07cf8b20d7dc2a4b . "./code.cg2js/cg-intersect.fasl")
(#xb467d309eafcfb947d9820ef5d82d5f0 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x0bfb6b5065929a2270679c745b7cd534 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#xf07506d6b52d228169351d32f665e3f2 . "./code.cg2js/cg-class-support.fasl")
(#xe1ea68c6d2dd9ff2dfb4a3b14650d514 . "./code.cg2js/cg-dialog-item.fasl")
(#xcb172d37a3014ab4611738a41ae857b0 . "./code.cg2js/cg-plot-widget.fasl")
(#x613e7d92bc0af4fc375208ce42b017c0 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xd06e897c41fb7f88f00d21c08c0fa49a . "./code.cg2js/cg-up-down-control.fasl")
(#xa8f5cc74702d696f5084b7a1cca6ff63 . "./code.cg2js/cg-rich-edit.fasl")
(#x611bfeac0825fbdeaedbc382694f1ed1 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xdf873a89fa326b1d37a8d4c8a18f81a1 . "./code.cg2js/cg-status-bar.fasl")
(#xc454d73238e688ab31dabf235355839c . "./code.cg2js/cg-class-grid.fasl")
(#xe9939dbc11c60e8492176ab4c7ff1c9a . "./code.cg2js/cg-wrap-string.fasl")
(#xecf9059ac6fe8927c54d5572aebf14ad . "./code.cg2js/cg-layout.fasl")
(#x8b54bb74c8c6726790932e23dfa6b1a7 . "./code.cg2js/cg-lisp-widget.fasl")
(#x8bd1eb41a6a80a8ff924cf9c3c9f1b7a . "./code.cg2js/cg-button.fasl")
(#x3a405f8bb5eae4bac53fba767486de7e . "./code.cg2js/cg-string-dialog.fasl")
(#xfb202cdcc2e02b342d26b91e3fa5c570 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xf25dff2c465b1daa55c67d2d4fcca96b . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x52177c9fc36a0994d0f2dc35c492aa71 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x4b40f86ab83a05f4ae0f02253ee5b927 . "./code.cg2js/cg-static-text.fasl")
(#xaa8e54c7129012cf409fab854e10817c . "./code.cg2js/cg-outline.fasl")
(#x0109b7ac8ba8e2fcf90c00163fb28233 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x87d999d1853c5ca771dcba2524caa9b4 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x6dbaba141af481edc656af1f2171bf1f . "./code.cg2js/cg-picture-widget.fasl")
(#x5a1e35e8b64483e30f68c0cdbbb1c3eb . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x4f369e5fcd40c0d4dc50c57fbb347cc6 . "./code.cg2js/cg-chart-widget.fasl")
(#x38c9d6af26b19c144502e01413eca7c7 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x79c33dd8e64947ed8ced1b26251471fc . "./code.cg2js/cg-lisp-text.fasl")
(#xa6b7a063a090df3740bc80ba12f8af09 . "./code.cg2js/cg-palette.fasl")
(#x603dd648698d9488a606ff48fe846d01 . "./code.cg2js/cg-property.fasl")
(#xbef55b2853ff007473aaae639932484a . "./code.cg2js/cg-intersect-posbox.fasl")
(#x8b7416ce5ded1901d16b4595c5dbdff7 . "./code.cg2js/cg-file-dialog.fasl")
(#x14ee0294046089b34969641b66257ef0 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x48355f338b2620e592f0bde684cfae63 . "./code.cg2js/cg-get-pixmap.fasl")
(#x80c90ae128a1963a10e1e47b098abafd . "./code.cg2js/cg-tray.fasl")
(#xbd8c2d0d82757ab971843e7c8b9bd869 . "./code.cg2js/cg-acache.fasl")
(#x261bd4f0f15f39ed4db7c88103e2fd56 . "./code.cg2js/cg-message-dialog.fasl")
(#x45d5d45c4d0ccdacb3ae64017b2475b3 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x12be84ae48eb394f363bb81323fb43bb . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x64a2196258a75cb45bc8babeacf5885d . "./code.cg2js/cg-tab-control.fasl")
(#x9aa67b92a937a5aa15bf762b963110c3 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x9af5b5f9fa77fe923df5d320a7f34843 . "./code.cg2js/cg-menu-tooltip.fasl")
(#xc53697b6d573f155806907d41d3aaa4a . "./code.cg2js/cg-find-string-dialog.fasl")
(#x43854ddf6bc5e1e50bb2ef69a10e48d0 . "./code.cg2js/cg-menu.fasl")
(#x16fb333d5bc0b5435bbc4c07b9775b99 . "./code.cg2js/cg-get-position.fasl")
(#xea9b5c19bd4d93d2b1509d4bcad192b0 . "./code.cg2js/cg-trackbar.fasl")
(#xc2ea09644f8ffb56326a877c4acec696 . "./code.cg2js/cg-selected-object.fasl")
(#xcff16471ad6bc17df71c9824fdf6854f . "./code.cg2js/cg-list-view.fasl")
(#xe3ed44386e7021e497a3f06b5091575b . "./code.cg2js/cg-html-widget.fasl")
(#x3924d19cdaed9bd9d0faba9508ec2469 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x238be5a596e45495a5414c6fb196eeeb . "./code.cg2js/cg-text-widget.fasl")
(#x9749bfb01cf939fa80ad02f32ac14d5a . "./code.idejs/ide-base.fasl")
(#x9adfd8cc94bd5bfc4e75ff154f6f27d8 . "./code.idejs/ide-process-browser.fasl")
(#x48b6c5cacc213724595138f4cb1e21ca . "./code.idejs/ide-patch-dialog.fasl")
(#xcbfd0c667086e714b5f32ff7e4feb161 . "./code.idejs/ide-keyshort.fasl")
(#x80052036b8e338080d736c14335b7648 . "./code.idejs/ide-window-list.fasl")
(#x8a22e86cf83ff108cf04183f3c99b57f . "./code.idejs/ide.fasl")
(#x149777c8d72371976966c29a42058068 . "./code.idejs/ide-editor.fasl")
(#xa11b2e78c318675c3e46a337db97f3bd . "./code.idejs/ide-clipboard.fasl")
(#x54ad92fabe29dabbc14db61fd09dc505 . "./code.idejs/ide-trace.fasl")
(#x1440f614c3e9af4b47c629c20692af16 . "./code.idejs/ide-class-browser.fasl")
(#xe3f532afdab263631300ba79c13d0f9c . "./code.idejs/ide-grapher.fasl")
(#x949713bf1b9262c0d97388eca86b47f7 . "./code.idejs/ide-apropos.fasl")
(#x589c726df736375a512cd0616f008aa5 . "./code.idejs/ide-navigator.fasl")
(#x238f83d0de154db5350047a61719f09d . "./code.idejs/ide-find-definitions.fasl")
(#x093fc665ad567029d838578e9349fc81 . "./code.idejs/ide-project.fasl")
(#x3f5cbcfa459c55652579b1bd1965bf9b . "./code.idejs/ide-class-grapher.fasl")
(#xc2309862994dd26e55f3fe0ee445c019 . "./code.idejs/ide-acache.fasl")
(#x9144b94ddc5d56133d1157ee6d5e7494 . "./code.idejs/ide-find-in-files.fasl")
(#x5539a154f3bf2aa841171a3dfa710282 . "./code.idejs/ide-inspector.fasl")
(#xe4c45b466e578b7e7c78eea8d300b694 . "./code.idejs/ide-intro.fasl")
(#x3dbaf7fa96f81cf7ec5c8c6b058d197d . "./code.idejs/ide-package-list.fasl")
(#xb05e71820358e216e8ea66e0000d4156 . "./code.idejs/ide-dev.fasl")
(#x1c62d7639a4b84689c79638d3cedf5bf . "./code.idejs/ide-form.fasl")
(#x4469068b47e063cb9811630bd05af4c2 . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
